//+--------------------------------------------------+
//| Trades ZigZag breakouts, filtered by the ATR MA  |
//|      and using Fibonacci profit targeting.       |
//+--------------------------------------------------+

#define SIGNAL_NONE 0
#define SIGNAL_BUY   1
#define SIGNAL_SELL  2
#define SIGNAL_CLOSEBUY 3
#define SIGNAL_CLOSESELL 4

#property copyright "Alexander C. Boyd"

extern string  BasicSettings        = "=======> EA Settings";
extern int     MagicNumber          = 1918271;
extern int     Slippage             = 3;
extern bool    EachTickMode         = true;
extern bool    SignalMail           = False;

extern string  SwingSettings        = "=======> ZigZag Settings";
extern int     Depth                = 15;

extern string  ATRSettings          = "=======> ATR Settings";
extern int     ATRPeriod            = 34;
extern int     BaseSMA              = 1;
extern int     SignalEMA            = 26;

extern string  SessionTimeSettings  = "=======> Time Settings";
extern int     GMTOffset            = 1;
extern int     SessionOpen          = 8;
extern int     SessionClose         = 18;

extern string  MoneyManagement            = "=======> Money Management Settings";
extern bool    UseProportionalLotSizing   = true;  // Use equity as a gauge of lot size
extern double  RiskFactor                 = 1;     // Risk level
extern double  Lots                       = 0.1;   // If using static lot sizing

extern string  OrderManagement            = "=======> Order Settings";
extern bool    XBarsTrailingStop          = true;
extern int     TrailingStopBars           = 3;
extern bool    UseFibonacciTargeting      = true;
extern int     FibonacciStopLossLevel     = 5;      // This is the integer (1-5) assigned to the fibonacci retracement, with the 0.0 level being 1, 23.6 being 2, etc... 
extern int     FibonacciTakeProfitLevel   = 1;      // This is the integer (1 or 2) assigned to the fibonacci expansion, with the 161.8 level being 1, 261.8 level being 2 
extern bool    UseTrailingStop            = false;
extern int     TrailingStop               = 185;
extern bool    UseStopLoss                = false;
extern int     StopLoss                   = 200;
extern bool    UseTakeProfit              = false;
extern int     TakeProfit                 = 500;

int BarCount;
int Current;
bool TickCheck = False;


//---------- initialization, technicalities, etc
int init() {BarCount = Bars; if (EachTickMode) Current = 0; else Current = 1; return(0);}
int deinit() {return(0);}

int start() {

   int Order = SIGNAL_NONE;
   int Ticket;
   double StopLossLongLevel, TakeProfitLongLevel, StopLossShortLevel, TakeProfitShortLevel;
   if (EachTickMode && Bars != BarCount) TickCheck = False;
   int Total = OrdersTotal();
   Order = SIGNAL_NONE;
   
//---------- Define variables

double   ATRHistogram   = iCustom(NULL, 0, "ATR MA", ATRPeriod, BaseSMA, SignalEMA, 3, 0);

double  LastBarHigh     = iHigh(NULL, 0, 1);  
double  LastBarLow      = iLow(NULL, 0, 1);  

double  CurrentLow = iMA(NULL, 0, 1, 0, MODE_SMA, PRICE_LOW, 0);
double  CurrentHigh = iMA(NULL, 0, 1, 0, MODE_SMA, PRICE_HIGH, 0);

double  PreviousLow = iMA(NULL, 0, 1, 0, MODE_SMA, PRICE_LOW, 1);
double  PreviousHigh = iMA(NULL, 0, 1, 0, MODE_SMA, PRICE_HIGH, 1);

      double SwingValue[3]={0,0,0};
      int Found = 0;
      int k = 0;
      while(Found < 3){
         if(iCustom(NULL, 0, "ZigZag", Depth, 5, 3, 0, k) != 0){
            SwingValue[Found] = iCustom(NULL , 0, "ZigZag", Depth, 5, 3, 0, k);
            Found++;
         }
         k++;
      }

Print("Recent ZigZag value: "+SwingValue[1]+", Second recent ZigZag value: "+SwingValue[2]+"");



//---------- Define trading signals

bool BuySignal, SellSignal;
/*
if (PreviousHigh < SwingValue[2] && CurrentHigh > SwingValue[2]) BuySignal = true;
if (PreviousLow > SwingValue[2] && CurrentLow < SwingValue[2]) SellSignal = true;
*/

if (SwingValue[0] > SwingValue[1] && SwingValue[1] < SwingValue[2])
   {
   if (CurrentHigh > SwingValue[2])
      {
      BuySignal = true;
      }
else if (SwingValue[0] < SwingValue[1] && SwingValue[1] > SwingValue[2])
   {
   if (CurrentLow < SwingValue[2])
      {
      SellSignal = true;
      }
   }
}


//---------- Get spread info

double Multiple = MathPow(10,Digits);
double Spread = ((Ask-Bid)*Multiple);


//---------- Define the time frame and ATR condition

if (ATRHistogram > 0 && Hour() >= (SessionOpen+GMTOffset) && Hour() <= (SessionClose+GMTOffset)) {
    bool TimeFrameOkay = true;}


//---------- Money Management

      if (UseProportionalLotSizing){double Lots = NormalizeDouble(AccountFreeMargin()*(0.00001*RiskFactor), 2);
      if (Lots < 0.01) Lots = MODE_MINLOT;}


//---------- Fibonacci Targeting

if(UseFibonacciTargeting)   {

    switch(FibonacciStopLossLevel)  {
        case 1:    FibonacciStopLossLevel = 0;          break;
        case 2:    FibonacciStopLossLevel = 0.23;       break;
        case 3:    FibonacciStopLossLevel = 0.375;      break;
        case 4:    FibonacciStopLossLevel = 0.49;       break;
        case 5:    FibonacciStopLossLevel = 0.60;       break;       }

    switch(FibonacciTakeProfitLevel)  {
        case 1:    FibonacciTakeProfitLevel = 1.6;      break;
        case 2:    FibonacciTakeProfitLevel = 2.59;     break;       }

    UseStopLoss = true;
    UseTakeProfit = true;
    
       double TakeProfitLong = SwingValue[1]+(SwingValue[1]-SwingValue[0])*FibonacciTakeProfitLevel;
       double TakeProfitShort = SwingValue[1]-(SwingValue[0]-SwingValue[1])*FibonacciTakeProfitLevel;
       double StopLossLong = SwingValue[1]-(SwingValue[1]-SwingValue[0])*FibonacciStopLossLevel;
       double StopLossShort = SwingValue[1]+(SwingValue[0]-SwingValue[1])*FibonacciStopLossLevel;
       
        }

//------- How to close trades

   bool IsTrade = False;

   for (int i = 0; i < Total; i ++) {
      OrderSelect(i, SELECT_BY_POS, MODE_TRADES);
      if(OrderType() <= OP_SELL &&  OrderSymbol() == Symbol()) {
         IsTrade = True;
         if(OrderType() == OP_BUY) {

//------- Exit buy signal



//------- Exit buy [end]

            if (Order == SIGNAL_CLOSEBUY && ((EachTickMode && !TickCheck) || (!EachTickMode && (Bars != BarCount)))) {
               OrderClose(OrderTicket(), OrderLots(), Bid, Slippage, MediumSeaGreen);
               if (!EachTickMode) BarCount = Bars;
               IsTrade = False;
               continue;
            }
            //Trailing stop
            if(UseTrailingStop && TrailingStop > 0) {                 
               if(Bid - OrderOpenPrice() > Point * TrailingStop) {
                  if(OrderStopLoss() < Bid - Point * TrailingStop) {
                     OrderModify(OrderTicket(), OrderOpenPrice(), Bid - Point * TrailingStop, OrderTakeProfit(), 0, MediumSeaGreen);
                     if (!EachTickMode) BarCount = Bars;
                     continue;
                  }
               }
            }
         } else {

//------- Exit sell signal



//------- Exit sell [end]

            if (Order == SIGNAL_CLOSESELL && ((EachTickMode && !TickCheck) || (!EachTickMode && (Bars != BarCount)))) {
               OrderClose(OrderTicket(), OrderLots(), Ask, Slippage, DarkOrange);
               if (!EachTickMode) BarCount = Bars;
               IsTrade = False;
               continue;
            }
            //Trailing stop
            if(UseTrailingStop && TrailingStop > 0) {                 
               if((OrderOpenPrice() - Ask) > (Point * TrailingStop)) {
                  if((OrderStopLoss() > (Ask + Point * TrailingStop)) || (OrderStopLoss() == 0)) {
                     OrderModify(OrderTicket(), OrderOpenPrice(), Ask + Point * TrailingStop, OrderTakeProfit(), 0, DarkOrange);
                     if (!EachTickMode) BarCount = Bars;
                     continue;
                  }
               }
            }
         }
      }
   }

//------- CONDITIONS FOR OPENING TRADES

   if (TimeFrameOkay && BuySignal) Order = SIGNAL_BUY;
   if (TimeFrameOkay && SellSignal) Order = SIGNAL_SELL;
   

//------- Enter buy

   if (Order == SIGNAL_BUY && ((EachTickMode && !TickCheck) || (!EachTickMode && (Bars != BarCount)))) {
      if(!IsTrade) {

         if (UseStopLoss) StopLossLongLevel = Ask - StopLossLong * Point; else StopLossLongLevel = 0.0;
         if (UseTakeProfit) TakeProfitLongLevel = Ask + TakeProfitLong * Point; else TakeProfitLongLevel = 0.0;

         Ticket = OrderSend(Symbol(), OP_BUY, Lots, Ask, Slippage, 0, 0, "Buy(#" + MagicNumber + ")", MagicNumber, 0, DodgerBlue);
         if(Ticket > 0) {

            if (OrderSelect(Ticket, SELECT_BY_TICKET, MODE_TRADES)) {
                Print("BUY order opened : ", OrderOpenPrice());
                
			} else {
				Print("Error opening BUY order : ", GetLastError());
			}
        }

         if (EachTickMode) TickCheck = True;
         if (!EachTickMode) BarCount = Bars;
         return(0);
      }
   }


//------- Enter sell

   if (Order == SIGNAL_SELL && ((EachTickMode && !TickCheck) || (!EachTickMode && (Bars != BarCount)))) {
      if(!IsTrade) {

         if (UseStopLoss) StopLossShortLevel = Bid + StopLossShort * Point; else StopLossShortLevel = 0.0;
         if (UseTakeProfit) TakeProfitShortLevel = Bid - TakeProfitShort * Point; else TakeProfitShortLevel = 0.0;

         Ticket = OrderSend(Symbol(), OP_SELL, Lots, Bid, Slippage, 0, 0, "Sell(#" + MagicNumber + ")", MagicNumber, 0, DeepPink);
         if(Ticket > 0) {

            if (OrderSelect(Ticket, SELECT_BY_TICKET, MODE_TRADES)) {
				Print("SELL order opened : ", OrderOpenPrice());
			} else {
				Print("Error opening SELL order : ", GetLastError());
			}
         }
         if (EachTickMode) TickCheck = True;
         if (!EachTickMode) BarCount = Bars;
         return(0);
      }
   }

for(int n = OrdersTotal()-1; n >= 0; n--)   {
  if(OrderSelect(i,SELECT_BY_POS,MODE_TRADES)==true)    {
  
//------- Exit Long
    if(OrderType()==OP_BUY)     {
      if(OrderProfit()>=TakeProfitLong*OrderLots()
      || OrderProfit()<=-StopLossLong*OrderLots()
      )     {
        OrderClose(OrderTicket(),OrderLots(),Bid,Slippage,White);
        }
    }
 }
 
//------- Exit Short
    if(OrderType()==OP_SELL)    {
      if(OrderProfit()>=TakeProfitShort*OrderLots()
      || OrderProfit()<=-StopLossShort*OrderLots()
      )     {
        OrderClose(OrderTicket(),OrderLots(),Ask,Slippage,White);
      }
    }
 }
   if (!EachTickMode) BarCount = Bars;

//------- Comment relevant/important information

string BreakoutBias = "None";

if (SwingValue[1] > SwingValue[0]) BreakoutBias = "Long"; 
if (SwingValue[1] < SwingValue[0]) BreakoutBias = "Short";
if (TimeFrameOkay == true) string TradingAllowed = "Active";
if (TimeFrameOkay == false) TradingAllowed = "Inactive";

double FreeMargin = NormalizeDouble(AccountFreeMargin(), 2);
double TotalProfit = NormalizeDouble(AccountProfit(), 2);

Comment(
        "Time Zone: "+TradingAllowed+" \n",
        "Breakout Bias: "+BreakoutBias+" \n",
        "Spread: "+Spread+" \n",
        "\n",
        "Available capital: "+FreeMargin+" \n",
        "Floating P/L: "+TotalProfit+" \n"
        );

   return(0);
}
//+------------------------------------------------------------------+

